@@ -1,6 +1,9 @@
 from __future__ import annotations
 
+from dataclasses import replace
+from decimal import Decimal
 from pathlib import Path
+import datetime as dt
 
 import duckdb
 import pytest
@@ -24,8 +27,8 @@ def test_cache_store_respects_configured_storage_root(tmp_path: Path) -> None:
     config_path = tmp_path / "config.toml"
     config_path.write_text(
         f"""
-[server]
-storage_root = "{storage_root.as_posix()}"
+[runtime]
+storage = "{storage_root.as_posix()}"
 """.strip(),
         encoding="utf-8",
     )
@@ -40,6 +43,39 @@ storage_root = "{storage_root.as_posix()}"
     assert expected_cache_dir.is_dir()
 
 
+def test_cache_route_signature_includes_constants(tmp_path: Path) -> None:
+    route = RouteDefinition(
+        id="sample",
+        path="/sample",
+        methods=["GET"],
+        raw_sql="SELECT source.table",
+        prepared_sql="SELECT source.table",
+        param_order=[],
+        params=[],
+        constants={"table": "source.table"},
+        constant_params={},
+        constant_types={"table": "IDENTIFIER"},
+        constant_param_types={},
+    )
+    alt_route = replace(
+        route,
+        constants={"table": "other.table"},
+        constant_types={"table": "IDENTIFIER"},
+    )
+    assert cache_mod._route_signature(route) != cache_mod._route_signature(alt_route)
+
+    store = cache_mod.CacheStore(tmp_path)
+    settings = cache_mod.CacheSettings(
+        enabled=True,
+        ttl_seconds=0,
+        rows_per_page=10,
+        enforce_page_size=False,
+    )
+    key1 = store.compute_key(route, {}, settings)
+    key2 = store.compute_key(alt_route, {}, settings)
+    assert key1.digest != key2.digest
+
+
 @pytest.mark.skipif(TestClient is None, reason="fastapi is not available")
 def test_cache_files_materialize_under_configured_storage_root(tmp_path: Path) -> None:
     src = tmp_path / "src"
@@ -49,8 +85,8 @@ def test_cache_files_materialize_under_configured_storage_root(tmp_path: Path) -
     config_path = tmp_path / "config.toml"
     config_path.write_text(
         f"""
-[server]
-storage_root = "{storage_root.as_posix()}"
+[runtime]
+storage = "{storage_root.as_posix()}"
 """.strip(),
         encoding="utf-8",
     )
@@ -231,7 +267,7 @@ def test_invariant_filter_uses_superset_cache(tmp_path: Path, monkeypatch: pytes
         "    ('gadget', 2, 2),\n"
         "    ('widget', 3, 3)\n"
         ") AS inventory(product_code, quantity, seq)\n"
-        "WHERE product_code = COALESCE({{ product_code }}, product_code)\n"
+        "WHERE product_code = COALESCE($product_code, product_code)\n"
         "ORDER BY seq\n"
         "```\n"
     )
@@ -296,7 +332,7 @@ def test_invariant_filter_case_insensitive_values(
         "    ('gadget', 2, 2),\n"
         "    ('widget', 3, 3)\n"
         ") AS inventory(product_code, quantity, seq)\n"
-        "WHERE product_code = COALESCE({{ product_code }}, product_code)\n"
+        "WHERE product_code = COALESCE($product_code, product_code)\n"
         "ORDER BY seq\n"
         "```\n"
     )
@@ -393,7 +429,7 @@ def test_invariant_filter_supports_null_requests(
         "    (NULL, 2),\n"
         "    ('gadget', 3)\n"
         ") AS inventory(product_code, seq)\n"
-        "WHERE product_code IS NOT DISTINCT FROM COALESCE(NULLIF({{ product_code }}, ''), product_code)\n"
+        "WHERE product_code IS NOT DISTINCT FROM COALESCE(NULLIF($product_code, ''), product_code)\n"
         "ORDER BY seq\n"
         "```\n"
     )
@@ -472,7 +508,7 @@ def test_invariant_combines_filtered_caches(tmp_path: Path, monkeypatch: pytest.
         "    ('gadget', 2, 2),\n"
         "    ('widget', 3, 3)\n"
         ") AS inventory(product_code, quantity, seq)\n"
-        "WHERE product_code = COALESCE({{ product_code }}, product_code)\n"
+        "WHERE product_code = COALESCE($product_code, product_code)\n"
         "ORDER BY seq\n"
         "```\n"
     )
@@ -548,7 +584,7 @@ def test_invariant_partial_cache_triggers_query(tmp_path: Path, monkeypatch: pyt
         "    ('gadget', 2, 2),\n"
         "    ('widget', 3, 3)\n"
         ") AS inventory(product_code, quantity, seq)\n"
-        "WHERE product_code = COALESCE({{ product_code }}, product_code)\n"
+        "WHERE product_code = COALESCE($product_code, product_code)\n"
         "ORDER BY seq\n"
         "```\n"
     )
@@ -737,7 +773,7 @@ def test_invariant_select_defaults_to_unique_values(tmp_path: Path) -> None:
         "    ('FIN', 'Finance', 'Payroll'),\n"
         "    ('MFG', 'Manufacturing', 'Assembly')\n"
         ") AS t(Code, Division, Department)\n"
-        "WHERE {{ division }} = '' OR Division = {{ division }}\n"
+        "WHERE $division = '' OR Division = $division\n"
         "ORDER BY Division\n"
         "```\n"
     )
@@ -796,8 +832,8 @@ def test_invariant_html_form_filters_after_numeric_selection(tmp_path: Path) ->
         "    (2024, 'Engineering', 'ENG-10'),\n"
         "    (2024, 'Operations', 'OPS-11')\n"
         ") AS t(Year, Division, Code)\n"
-        "WHERE ({{ year }} = '' OR Year = CAST({{ year }} AS INTEGER))\n"
-        "  AND ({{ division }} = '' OR Division = {{ division }})\n"
+        "WHERE ($year = '' OR Year = CAST($year AS INTEGER))\n"
+        "  AND ($division = '' OR Division = $division)\n"
         "ORDER BY Year, Division\n"
         "```\n"
     )
@@ -924,7 +960,7 @@ def test_invariant_unique_values_respect_filter_context(tmp_path: Path) -> None:
         "    ('Finance', 'Payroll'),\n"
         "    ('Manufacturing', 'Assembly Line 1')\n"
         ") AS t(Division, Department)\n"
-        "WHERE {{ division }} = '' OR Division = {{ division }}\n"
+        "WHERE $division = '' OR Division = $division\n"
         "ORDER BY Division, Department\n"
         "```\n"
     )
@@ -998,6 +1034,87 @@ def test_canonicalize_invariant_value_normalizes_negative_zero() -> None:
     )
 
 
+def test_canonicalize_invariant_value_handles_temporal_types() -> None:
+    setting = cache_mod.InvariantFilterSetting(param="when", column="when")
+
+    date_value = dt.date(2024, 1, 2)
+    naive_dt = dt.datetime(2024, 1, 2, 3, 4, 5)
+    aware_dt = dt.datetime(2024, 1, 2, 3, 4, 5, tzinfo=dt.timezone.utc)
+
+    assert cache_mod.canonicalize_invariant_value(date_value, setting) == "date:2024-01-02"
+    assert cache_mod.canonicalize_invariant_value(naive_dt, setting) == "datetime:2024-01-02T03:04:05"
+    assert (
+        cache_mod.canonicalize_invariant_value(aware_dt, setting)
+        == "datetime:2024-01-02T03:04:05Z"
+    )
+
+
+def test_prepare_invariant_filter_values_parses_temporal_strings() -> None:
+    date_setting = cache_mod.InvariantFilterSetting(param="day", column="day")
+    date_column = pa.chunked_array(
+        [[dt.date(2024, 1, 1), dt.date(2024, 1, 2)]],
+        type=pa.date32(),
+    )
+
+    values, include_null, use_casefold = cache_mod._prepare_invariant_filter_values(
+        ["2024-01-02", dt.datetime(2024, 1, 3, 9, 15, tzinfo=dt.timezone.utc)],
+        date_column,
+        date_setting,
+    )
+
+    assert values == [dt.date(2024, 1, 2), dt.date(2024, 1, 3)]
+    assert include_null is False
+    assert use_casefold is False
+
+    ts_setting = cache_mod.InvariantFilterSetting(param="event", column="event")
+    ts_column = pa.chunked_array(
+        [[dt.datetime(2024, 1, 1, 8, 0, tzinfo=dt.timezone.utc)]],
+        type=pa.timestamp("us", tz="UTC"),
+    )
+
+    ts_values, ts_include_null, ts_use_casefold = cache_mod._prepare_invariant_filter_values(
+        ["2024-01-02T03:04:05Z", dt.date(2024, 1, 2)],
+        ts_column,
+        ts_setting,
+    )
+
+    assert len(ts_values) == 2
+    assert isinstance(ts_values[0], dt.datetime)
+    assert ts_values[0].tzinfo is not None
+    assert ts_values[0] == dt.datetime(2024, 1, 2, 3, 4, 5, tzinfo=dt.timezone.utc)
+    assert isinstance(ts_values[1], dt.datetime)
+    assert ts_values[1].tzinfo is not None
+    assert ts_values[1].replace(tzinfo=None) == dt.datetime(2024, 1, 2, 0, 0)
+    assert ts_include_null is False
+    assert ts_use_casefold is False
+
+
+def test_prepare_invariant_filter_values_handles_tz_inputs_for_naive_timestamp_columns() -> None:
+    setting = cache_mod.InvariantFilterSetting(param="event", column="event")
+    column = pa.chunked_array(
+        [[dt.datetime(2024, 1, 2, 3, 4, 5)]],
+        type=pa.timestamp("us"),
+    )
+
+    values, include_null, use_casefold = cache_mod._prepare_invariant_filter_values(
+        ["2024-01-02T03:04:05Z", dt.datetime(2024, 1, 2, 3, 4, 5, tzinfo=dt.timezone.utc)],
+        column,
+        setting,
+    )
+
+    assert all(isinstance(item, dt.datetime) for item in values)
+    assert all(item.tzinfo is None for item in values)
+    assert values[0] == dt.datetime(2024, 1, 2, 3, 4, 5)
+    assert values[1] == dt.datetime(2024, 1, 2, 3, 4, 5)
+    assert include_null is False
+    assert use_casefold is False
+
+    # Ensure the coerced values can be materialized as an Arrow array that
+    # matches the naive timestamp column type.
+    pa_values = pa.array(values, type=column.type)
+    assert pa_values.type == column.type
+
+
 def test_canonicalize_invariant_mapping_collapses_decimal_variants() -> None:
     setting = cache_mod.InvariantFilterSetting(param="amount", column="amount")
     mapping = cache_mod.canonicalize_invariant_mapping(
@@ -1093,7 +1210,7 @@ def test_invariant_unique_values_merge_with_static_options(tmp_path: Path) -> No
         "    ('ENG', 'Engineering'),\n"
         "    ('FIN', 'Finance')\n"
         ") AS t(Code, Division)\n"
-        "WHERE {{ division }} = '' OR Division = {{ division }}\n"
+        "WHERE $division = '' OR Division = $division\n"
         "ORDER BY Division\n"
         "```\n"
     )
@@ -1113,4 +1230,3 @@ def test_invariant_unique_values_merge_with_static_options(tmp_path: Path) -> No
     assert "<option value='Engineering'>Engineering</option>" in text
     assert "<option value='Finance'>Finance</option>" in text
     assert "<option value='Other'>Custom</option>" in text
-from decimal import Decimal
