@@ -3,6 +3,7 @@ from __future__ import annotations
 import datetime as dt
 from pathlib import Path
 
+import os
 import sys
 import types
 
@@ -11,40 +12,41 @@ import pytest
 from webbed_duck import cli
 
 
-def test_build_source_fingerprint_missing_directory(tmp_path: Path) -> None:
+def test_build_watch_snapshot_missing_directory(tmp_path: Path) -> None:
     missing = tmp_path / "not_there"
-    fingerprint = cli.build_source_fingerprint(missing)
-    assert isinstance(fingerprint, cli.SourceFingerprint)
-    assert dict(fingerprint.files) == {}
+    snapshot = cli.build_watch_snapshot(missing, None)
+    assert isinstance(snapshot, cli.WatchSnapshot)
+    assert dict(snapshot.routes) == {}
+    assert dict(snapshot.plugins) == {}
 
 
-def test_build_source_fingerprint_detects_changes(tmp_path: Path) -> None:
+def test_build_watch_snapshot_detects_changes(tmp_path: Path) -> None:
     src = tmp_path / "src"
     src.mkdir()
     (src / "demo.toml").write_text("id='demo'\n", encoding="utf-8")
     sql_path = src / "demo.sql"
     sql_path.write_text("SELECT 1;\n", encoding="utf-8")
 
-    initial = cli.build_source_fingerprint(src)
-    assert "demo.toml" in initial.files
-    assert "demo.sql" in initial.files
+    initial = cli.build_watch_snapshot(src, None)
+    assert "demo.toml" in initial.routes
+    assert "demo.sql" in initial.routes
 
     sql_path.write_text("SELECT 2; -- changed\n", encoding="utf-8")
-    updated = cli.build_source_fingerprint(src)
-    assert initial.has_changed(updated)
-    assert updated.has_changed(initial)
+    updated = cli.build_watch_snapshot(src, None)
+    assert initial.routes_changed(updated)
+    assert updated.routes_changed(initial)
 
 
-def test_build_source_fingerprint_custom_patterns(tmp_path: Path) -> None:
+def test_build_watch_snapshot_custom_patterns(tmp_path: Path) -> None:
     src = tmp_path / "src"
     src.mkdir()
     (src / "include.md").write_text("docs", encoding="utf-8")
 
-    default = cli.build_source_fingerprint(src)
-    assert "include.md" in default.files
+    default = cli.build_watch_snapshot(src, None)
+    assert "include.md" in default.routes
 
-    custom = cli.build_source_fingerprint(src, patterns=("*.md",))
-    assert set(custom.files) == {"include.md"}
+    custom = cli.build_watch_snapshot(src, None, route_patterns=("*.md",))
+    assert set(custom.routes) == {"include.md"}
 
 
 def test_parse_param_assignments_handles_invalid_pairs() -> None:
@@ -81,17 +83,21 @@ def test_start_watcher_clamps_interval(
 
     monkeypatch.setattr(cli.threading, "Thread", fake_thread)
 
+    plugins_dir = tmp_path / "plugins"
+    plugins_dir.mkdir(exist_ok=True)
     app = types.SimpleNamespace(state=types.SimpleNamespace())
-    stop_event, thread = cli._start_watcher(app, tmp_path, tmp_path, 0.05)
+    stop_event, thread = cli._start_watcher(app, tmp_path, tmp_path, plugins_dir, 0.05)
 
     assert isinstance(stop_event, cli.threading.Event)
     assert isinstance(thread, _FakeThread)
     assert captured["name"] == "webbed-duck-watch"
     assert captured["daemon"] is True
     args = captured["args"]
-    assert isinstance(args, tuple) and len(args) == 5
+    assert isinstance(args, tuple) and len(args) == 7
     assert args[1] == tmp_path and args[2] == tmp_path
-    assert args[3] == pytest.approx(0.2)
+    assert args[3] == plugins_dir
+    assert args[4] == pytest.approx(0.2)
+    assert args[6] is cli.compile_routes
     out = capsys.readouterr().out
     assert "[webbed-duck] Watching" in out
     assert "interval=0.20s" in out
@@ -105,11 +111,11 @@ def test_watch_iteration_skips_when_fingerprint_unchanged(tmp_path: Path) -> Non
     (src / "demo.toml").write_text("id='demo'\n", encoding="utf-8")
     (src / "demo.sql").write_text("SELECT 1;\n", encoding="utf-8")
 
-    snapshot = cli.build_source_fingerprint(src)
+    snapshot = cli.build_watch_snapshot(src, None)
     app = object()
     called: dict[str, object] = {}
 
-    def fake_reload(app_arg, source_dir, build_dir):  # type: ignore[no-untyped-def]
+    def fake_reload(app_arg, source_dir, build_dir, **_kwargs):  # type: ignore[no-untyped-def]
         called["app"] = app_arg
         return 5
 
@@ -117,12 +123,14 @@ def test_watch_iteration_skips_when_fingerprint_unchanged(tmp_path: Path) -> Non
         app,
         src,
         build,
-        snapshot,
+        plugins_dir=None,
+        snapshot=snapshot,
         compile_and_reload=fake_reload,
     )
 
     assert count == 0
-    assert updated is snapshot
+    assert updated.routes == snapshot.routes
+    assert updated.plugins == snapshot.plugins
     assert called == {}
 
 
@@ -136,13 +144,13 @@ def test_watch_iteration_triggers_reload_on_change(tmp_path: Path) -> None:
     toml_path.write_text("id='demo'\n", encoding="utf-8")
     sql_path.write_text("SELECT 1;\n", encoding="utf-8")
 
-    snapshot = cli.build_source_fingerprint(src)
+    snapshot = cli.build_watch_snapshot(src, None)
     sql_path.write_text("SELECT 2; -- change\n", encoding="utf-8")
 
     app = object()
     calls: dict[str, object] = {}
 
-    def fake_reload(app_arg, source_dir, build_dir):  # type: ignore[no-untyped-def]
+    def fake_reload(app_arg, source_dir, build_dir, **_kwargs):  # type: ignore[no-untyped-def]
         calls["app"] = app_arg
         calls["source"] = source_dir
         calls["build"] = build_dir
@@ -152,12 +160,13 @@ def test_watch_iteration_triggers_reload_on_change(tmp_path: Path) -> None:
         app,
         src,
         build,
-        snapshot,
+        plugins_dir=None,
+        snapshot=snapshot,
         compile_and_reload=fake_reload,
     )
 
     assert count == 2
-    assert updated.has_changed(snapshot)
+    assert updated.routes_changed(snapshot)
     assert calls == {"app": app, "source": src, "build": build}
 
 
@@ -208,16 +217,22 @@ def test_cmd_perf_reports_stats(monkeypatch: pytest.MonkeyPatch, capsys: pytest.
 def test_cmd_compile_reports_count(monkeypatch: pytest.MonkeyPatch, capsys: pytest.CaptureFixture[str]) -> None:
     captured: dict[str, object] = {}
 
-    def fake_compile(source: str | Path, build: str | Path) -> list[str]:
+    def fake_compile(source: str | Path, build: str | Path, **kwargs: object) -> list[str]:
         captured["source"] = source
         captured["build"] = build
+        captured["kwargs"] = kwargs
         return ["a", "b", "c"]
 
     monkeypatch.setattr(cli, "compile_routes", fake_compile)
 
-    code = cli._cmd_compile("src", "build")
+    code = cli._cmd_compile("src", "build", None)
     assert code == 0
-    assert captured == {"source": "src", "build": "build"}
+    assert captured["source"] == "src"
+    assert captured["build"] == "build"
+    kwargs = captured["kwargs"]
+    assert kwargs["server_constants"] == {}
+    assert kwargs["server_secrets"] == {}
+    assert Path(kwargs["plugins_dir"]) == Path(os.environ["WEBBED_DUCK_PLUGINS_DIR"])
     out = capsys.readouterr().out.strip()
     assert out == "Compiled 3 route(s) to build"
 
@@ -286,6 +301,8 @@ def test_cmd_serve_auto_compile_and_watch(
     build_dir = tmp_path / "build"
     build_dir.mkdir()
 
+    plugins_dir = tmp_path / "plugins"
+    plugins_dir.mkdir(exist_ok=True)
     server_config = types.SimpleNamespace(
         build_dir=build_dir,
         source_dir=None,
@@ -294,16 +311,29 @@ def test_cmd_serve_auto_compile_and_watch(
         watch_interval=1.5,
         host="127.0.0.1",
         port=8000,
+        constants={},
+        secrets={},
+        plugins_dir=plugins_dir,
+    )
+    storage_root = tmp_path / "storage"
+    storage_root.mkdir()
+    config_obj = types.SimpleNamespace(
+        server=server_config,
+        runtime=types.SimpleNamespace(storage=storage_root),
     )
-    config_obj = types.SimpleNamespace(server=server_config)
 
     monkeypatch.setattr(cli, "load_config", lambda path: config_obj)
 
     compiled_routes: list[Path] = []
 
-    def fake_compile(source: Path, build: Path) -> list[str]:
+    def fake_compile(source: Path, build: Path, **kwargs: object) -> list[str]:
         compiled_routes.append(source)
         assert build == build_dir
+        assert kwargs == {
+            "plugins_dir": plugins_dir,
+            "server_constants": {},
+            "server_secrets": {},
+        }
         return ["route"]
 
     monkeypatch.setattr(cli, "compile_routes", fake_compile)
@@ -330,11 +360,21 @@ def test_cmd_serve_auto_compile_and_watch(
         def join(self, timeout: float | None = None) -> None:
             self.join_timeout = timeout
 
-    def fake_start_watcher(app, source: Path, build: Path, interval: float):  # type: ignore[no-untyped-def]
+    def fake_start_watcher(
+        app,
+        source: Path,
+        build: Path,
+        plugins: Path | None,
+        interval: float,
+        *,
+        compile_fn=cli.compile_routes,
+    ):  # type: ignore[no-untyped-def]
         watcher_calls["app"] = app
         watcher_calls["source"] = source
         watcher_calls["build"] = build
+        watcher_calls["plugins"] = plugins
         watcher_calls["interval"] = interval
+        watcher_calls["compile_fn"] = compile_fn
         stop = _Stop()
         thread = _Thread()
         watcher_calls["stop"] = stop
@@ -345,15 +385,20 @@ def test_cmd_serve_auto_compile_and_watch(
 
     app_obj = types.SimpleNamespace()
     monkeypatch.setattr("webbed_duck.server.app.create_app", lambda routes, config: app_obj)
+    monkeypatch.setattr(
+        "webbed_duck.server.preferred_uvicorn_http_implementation",
+        lambda: "auto",
+    )
 
     run_calls: dict[str, object] = {}
 
     class _FakeUvicorn:
         @staticmethod
-        def run(app, host: str, port: int) -> None:  # type: ignore[no-untyped-def]
+        def run(app, host: str, port: int, **kwargs) -> None:  # type: ignore[no-untyped-def]
             run_calls["app"] = app
             run_calls["host"] = host
             run_calls["port"] = port
+            run_calls.update(kwargs)
 
     monkeypatch.setitem(sys.modules, "uvicorn", _FakeUvicorn)
 
@@ -374,10 +419,16 @@ def test_cmd_serve_auto_compile_and_watch(
     assert compiled_routes == [source_dir]
     assert watcher_calls["source"] == source_dir
     assert watcher_calls["build"] == build_dir
+    assert watcher_calls["plugins"] == plugins_dir
     assert watcher_calls["interval"] == 1.5
     assert watcher_calls["stop"].set_called is True
     assert watcher_calls["thread"].join_timeout == 2
-    assert run_calls == {"app": app_obj, "host": "127.0.0.1", "port": 8000}
+    assert run_calls == {
+        "app": app_obj,
+        "host": "127.0.0.1",
+        "port": 8000,
+        "http": "auto",
+    }
     err = capsys.readouterr().err
     assert "Auto-compile" not in err
 
@@ -390,6 +441,8 @@ def test_cmd_serve_watch_interval_clamp(
     build_dir = tmp_path / "build"
     build_dir.mkdir()
 
+    plugins_dir = tmp_path / "plugins"
+    plugins_dir.mkdir(exist_ok=True)
     server_config = types.SimpleNamespace(
         build_dir=build_dir,
         source_dir=source_dir,
@@ -398,8 +451,16 @@ def test_cmd_serve_watch_interval_clamp(
         watch_interval=1.0,
         host="127.0.0.1",
         port=8000,
+        constants={},
+        secrets={},
+        plugins_dir=plugins_dir,
+    )
+    storage_root = tmp_path / "storage"
+    storage_root.mkdir()
+    config_obj = types.SimpleNamespace(
+        server=server_config,
+        runtime=types.SimpleNamespace(storage=storage_root),
     )
-    config_obj = types.SimpleNamespace(server=server_config)
 
     monkeypatch.setattr(cli, "load_config", lambda path: config_obj)
     monkeypatch.setattr(cli, "compile_routes", lambda *_args, **_kwargs: [])
@@ -423,8 +484,18 @@ def test_cmd_serve_watch_interval_clamp(
         def join(self, timeout: float | None = None) -> None:
             self.join_timeout = timeout
 
-    def fake_start_watcher(app, source: Path, build: Path, interval: float):  # type: ignore[no-untyped-def]
+    def fake_start_watcher(
+        app,
+        source: Path,
+        build: Path,
+        plugins: Path | None,
+        interval: float,
+        *,
+        compile_fn=cli.compile_routes,
+    ):  # type: ignore[no-untyped-def]
+        recorded["plugins"] = plugins
         recorded["interval"] = interval
+        recorded["compile_fn"] = compile_fn
         stop = _Stop()
         thread = _Thread()
         recorded["stop"] = stop
@@ -435,12 +506,17 @@ def test_cmd_serve_watch_interval_clamp(
 
     app_obj = types.SimpleNamespace()
     monkeypatch.setattr("webbed_duck.server.app.create_app", lambda routes, config: app_obj)
+    monkeypatch.setattr(
+        "webbed_duck.server.preferred_uvicorn_http_implementation",
+        lambda: "auto",
+    )
 
     class _FakeUvicorn:
         @staticmethod
-        def run(app, host: str, port: int) -> None:  # type: ignore[no-untyped-def]
+        def run(app, host: str, port: int, **kwargs) -> None:  # type: ignore[no-untyped-def]
             recorded["host"] = host
             recorded["port"] = port
+            recorded.update(kwargs)
 
     monkeypatch.setitem(sys.modules, "uvicorn", _FakeUvicorn)
 
@@ -458,12 +534,13 @@ def test_cmd_serve_watch_interval_clamp(
 
     code = cli._cmd_serve(args)
     assert code == 0
+    assert recorded["plugins"] == plugins_dir
     assert recorded["interval"] == pytest.approx(cli.WATCH_INTERVAL_MIN)
     assert recorded["stop"].called is True
     assert recorded["thread"].join_timeout == 2
 
 
-def test_cmd_serve_config_watch_interval_clamped(
+def test_cmd_serve_forces_h11_on_windows_py313(
     monkeypatch: pytest.MonkeyPatch, tmp_path: Path
 ) -> None:
     source_dir = tmp_path / "src"
@@ -471,57 +548,51 @@ def test_cmd_serve_config_watch_interval_clamped(
     build_dir = tmp_path / "build"
     build_dir.mkdir()
 
+    plugins_dir = tmp_path / "plugins"
+    plugins_dir.mkdir(exist_ok=True)
     server_config = types.SimpleNamespace(
         build_dir=build_dir,
         source_dir=source_dir,
         auto_compile=False,
-        watch=True,
-        watch_interval=0.05,
+        watch=False,
+        watch_interval=1.0,
         host="127.0.0.1",
         port=8000,
+        constants={},
+        secrets={},
+        plugins_dir=plugins_dir,
+    )
+    storage_root = tmp_path / "storage"
+    storage_root.mkdir()
+    config_obj = types.SimpleNamespace(
+        server=server_config,
+        runtime=types.SimpleNamespace(storage=storage_root),
     )
-    config_obj = types.SimpleNamespace(server=server_config)
 
     monkeypatch.setattr(cli, "load_config", lambda path: config_obj)
     monkeypatch.setattr(cli, "compile_routes", lambda *_args, **_kwargs: [])
     monkeypatch.setattr(
         "webbed_duck.core.routes.load_compiled_routes", lambda build: ["route"]
     )
+    monkeypatch.setattr(
+        "webbed_duck.server.preferred_uvicorn_http_implementation",
+        lambda: "h11",
+    )
+
+    app_obj = types.SimpleNamespace()
+    monkeypatch.setattr(
+        "webbed_duck.server.app.create_app", lambda routes, config: app_obj
+    )
 
     recorded: dict[str, object] = {}
 
-    class _Stop:
-        def __init__(self) -> None:
-            self.called = False
-
-        def set(self) -> None:
-            self.called = True
-
-    class _Thread:
-        def __init__(self) -> None:
-            self.join_timeout: float | None = None
-
-        def join(self, timeout: float | None = None) -> None:
-            self.join_timeout = timeout
-
-    def fake_start_watcher(app, source: Path, build: Path, interval: float):  # type: ignore[no-untyped-def]
-        recorded["interval"] = interval
-        stop = _Stop()
-        thread = _Thread()
-        recorded["stop"] = stop
-        recorded["thread"] = thread
-        return stop, thread
-
-    monkeypatch.setattr(cli, "_start_watcher", fake_start_watcher)
-
-    app_obj = types.SimpleNamespace()
-    monkeypatch.setattr("webbed_duck.server.app.create_app", lambda routes, config: app_obj)
-
     class _FakeUvicorn:
         @staticmethod
-        def run(app, host: str, port: int) -> None:  # type: ignore[no-untyped-def]
+        def run(app, host: str, port: int, **kwargs) -> None:  # type: ignore[no-untyped-def]
+            recorded["app"] = app
             recorded["host"] = host
             recorded["port"] = port
+            recorded.update(kwargs)
 
     monkeypatch.setitem(sys.modules, "uvicorn", _FakeUvicorn)
 
@@ -539,6 +610,116 @@ def test_cmd_serve_config_watch_interval_clamped(
 
     code = cli._cmd_serve(args)
     assert code == 0
+    assert recorded == {
+        "app": app_obj,
+        "host": "127.0.0.1",
+        "port": 8000,
+        "http": "h11",
+    }
+
+
+def test_cmd_serve_config_watch_interval_clamped(
+    monkeypatch: pytest.MonkeyPatch, tmp_path: Path
+) -> None:
+    source_dir = tmp_path / "src"
+    source_dir.mkdir()
+    build_dir = tmp_path / "build"
+    build_dir.mkdir()
+
+    plugins_dir = tmp_path / "plugins"
+    plugins_dir.mkdir(exist_ok=True)
+    server_config = types.SimpleNamespace(
+        build_dir=build_dir,
+        source_dir=source_dir,
+        auto_compile=False,
+        watch=True,
+        watch_interval=0.05,
+        host="127.0.0.1",
+        port=8000,
+        constants={},
+        secrets={},
+        plugins_dir=plugins_dir,
+    )
+    storage_root = tmp_path / "storage"
+    storage_root.mkdir()
+    config_obj = types.SimpleNamespace(
+        server=server_config,
+        runtime=types.SimpleNamespace(storage=storage_root),
+    )
+
+    monkeypatch.setattr(cli, "load_config", lambda path: config_obj)
+    monkeypatch.setattr(cli, "compile_routes", lambda *_args, **_kwargs: [])
+    monkeypatch.setattr(
+        "webbed_duck.core.routes.load_compiled_routes", lambda build: ["route"]
+    )
+
+    recorded: dict[str, object] = {}
+
+    class _Stop:
+        def __init__(self) -> None:
+            self.called = False
+
+        def set(self) -> None:
+            self.called = True
+
+    class _Thread:
+        def __init__(self) -> None:
+            self.join_timeout: float | None = None
+
+        def join(self, timeout: float | None = None) -> None:
+            self.join_timeout = timeout
+
+    def fake_start_watcher(
+        app,
+        source: Path,
+        build: Path,
+        plugins: Path | None,
+        interval: float,
+        *,
+        compile_fn=cli.compile_routes,
+    ):  # type: ignore[no-untyped-def]
+        recorded["plugins"] = plugins
+        recorded["interval"] = interval
+        recorded["compile_fn"] = compile_fn
+        stop = _Stop()
+        thread = _Thread()
+        recorded["stop"] = stop
+        recorded["thread"] = thread
+        return stop, thread
+
+    monkeypatch.setattr(cli, "_start_watcher", fake_start_watcher)
+
+    app_obj = types.SimpleNamespace()
+    monkeypatch.setattr("webbed_duck.server.app.create_app", lambda routes, config: app_obj)
+    monkeypatch.setattr(
+        "webbed_duck.server.preferred_uvicorn_http_implementation",
+        lambda: "auto",
+    )
+
+    class _FakeUvicorn:
+        @staticmethod
+        def run(app, host: str, port: int, **kwargs) -> None:  # type: ignore[no-untyped-def]
+            recorded["host"] = host
+            recorded["port"] = port
+            recorded.update(kwargs)
+
+    monkeypatch.setitem(sys.modules, "uvicorn", _FakeUvicorn)
+
+    args = types.SimpleNamespace(
+        build=None,
+        source=None,
+        config="config.toml",
+        host=None,
+        port=None,
+        no_auto_compile=False,
+        watch=False,
+        no_watch=False,
+        watch_interval=None,
+    )
+
+    code = cli._cmd_serve(args)
+    assert code == 0
+    assert recorded["plugins"] == plugins_dir
     assert recorded["interval"] == pytest.approx(cli.WATCH_INTERVAL_MIN)
     assert recorded["stop"].called is True
     assert recorded["thread"].join_timeout == 2
@@ -551,6 +732,8 @@ def test_cmd_serve_auto_compile_failure_reports_error(
     build_dir = tmp_path / "build"
     build_dir.mkdir()
 
+    plugins_dir = tmp_path / "plugins"
+    plugins_dir.mkdir(exist_ok=True)
     server_config = types.SimpleNamespace(
         build_dir=build_dir,
         source_dir=source_dir,
@@ -559,8 +742,16 @@ def test_cmd_serve_auto_compile_failure_reports_error(
         watch_interval=1.0,
         host="127.0.0.1",
         port=8000,
+        constants={},
+        secrets={},
+        plugins_dir=plugins_dir,
+    )
+    storage_root = tmp_path / "storage"
+    storage_root.mkdir()
+    config_obj = types.SimpleNamespace(
+        server=server_config,
+        runtime=types.SimpleNamespace(storage=storage_root),
     )
-    config_obj = types.SimpleNamespace(server=server_config)
 
     monkeypatch.setattr(cli, "load_config", lambda path: config_obj)
 
