@@ -3,6 +3,7 @@ from __future__ import annotations
 
 import argparse
 import datetime
+import functools
 import statistics
 import sys
 import threading
@@ -11,24 +12,36 @@ from dataclasses import dataclass
 from pathlib import Path
 from typing import Mapping, Sequence
 
-from .config import load_config
+from .config import ConfigError, load_config
 from .core.compiler import compile_routes
 from .core.incremental import run_incremental
 from .core.local import run_route
+from .runtime.paths import get_storage
 
 WATCH_INTERVAL_MIN = 0.2
 
 
 @dataclass(frozen=True)
-class SourceFingerprint:
-    """Filesystem fingerprint for a route source directory."""
+class WatchSnapshot:
+    """Filesystem fingerprint for route sources and plugin directories."""
 
-    files: Mapping[str, tuple[float, int]]
+    routes: Mapping[str, tuple[float, int]]
+    plugins: Mapping[str, tuple[float, int]]
 
-    def has_changed(self, other: "SourceFingerprint") -> bool:
-        """Return ``True`` when ``other`` differs from this snapshot."""
+    def routes_changed(self, other: "WatchSnapshot") -> bool:
+        return dict(self.routes) != dict(other.routes)
 
-        return dict(self.files) != dict(other.files)
+    def plugin_changes(self, other: "WatchSnapshot") -> set[str]:
+        current = dict(self.plugins)
+        updated = dict(other.plugins)
+        changed: set[str] = set()
+        for key, value in updated.items():
+            if key not in current or current[key] != value:
+                changed.add(key)
+        for key in current:
+            if key not in updated:
+                changed.add(key)
+        return changed
 
 
 @dataclass(frozen=True)
@@ -72,6 +85,11 @@ def main(argv: Sequence[str] | None = None) -> int:
         help="Directory containing TOML/SQL route sidecars",
     )
     compile_parser.add_argument("--build", default="routes_build", help="Destination directory for compiled routes")
+    compile_parser.add_argument(
+        "--config",
+        default=None,
+        help="Optional configuration file providing shared constants and secrets",
+    )
 
     serve_parser = subparsers.add_parser("serve", help="Run the development server")
     serve_parser.add_argument("--build", default=None, help="Directory containing compiled routes")
@@ -106,7 +124,7 @@ def main(argv: Sequence[str] | None = None) -> int:
 
     args = parser.parse_args(argv)
     if args.command == "compile":
-        return _cmd_compile(args.source, args.build)
+        return _cmd_compile(args.source, args.build, args.config)
     if args.command == "serve":
         return _cmd_serve(args)
     if args.command == "run-incremental":
@@ -118,8 +136,20 @@ def main(argv: Sequence[str] | None = None) -> int:
     return 1
 
 
-def _cmd_compile(source: str, build: str) -> int:
-    compiled = compile_routes(source, build)
+def _cmd_compile(source: str, build: str, config_path: str | None) -> int:
+    try:
+        config = load_config(Path(config_path)) if config_path else load_config(None)
+    except ConfigError as exc:
+        print(f"[webbed-duck] ERROR: {exc}", file=sys.stderr)
+        return 2
+
+    compiled = compile_routes(
+        source,
+        build,
+        plugins_dir=config.server.plugins_dir,
+        server_constants=config.server.constants,
+        server_secrets=config.server.secrets,
+    )
     print(f"Compiled {len(compiled)} route(s) to {build}")
     return 0
 
@@ -128,7 +158,13 @@ def _cmd_serve(args: argparse.Namespace) -> int:
     from .core.routes import load_compiled_routes
     from .server.app import create_app
 
-    config = load_config(args.config)
+    try:
+        config = load_config(Path(args.config))
+    except ConfigError as exc:
+        print(f"[webbed-duck] ERROR: {exc}", file=sys.stderr)
+        return 2
+
+    print(f"[webbed-duck] Using storage: {get_storage(config)}")
 
     build_dir = Path(args.build) if args.build else Path(config.server.build_dir)
     source_dir = Path(args.source) if args.source else config.server.source_dir
@@ -151,9 +187,16 @@ def _cmd_serve(args: argparse.Namespace) -> int:
     if args.watch_interval is not None:
         watch_interval = max(WATCH_INTERVAL_MIN, float(args.watch_interval))
 
+    compile_fn = functools.partial(
+        compile_routes,
+        plugins_dir=config.server.plugins_dir,
+        server_constants=config.server.constants,
+        server_secrets=config.server.secrets,
+    )
+
     if auto_compile and source_dir is not None:
         try:
-            compiled = compile_routes(source_dir, build_dir)
+            compiled = compile_fn(source_dir, build_dir)
         except FileNotFoundError as exc:
             print(f"[webbed-duck] Auto-compile skipped: {exc}", file=sys.stderr)
         except Exception as exc:  # pragma: no cover - runtime safeguard
@@ -173,17 +216,31 @@ def _cmd_serve(args: argparse.Namespace) -> int:
     stop_event: threading.Event | None = None
     watch_thread: threading.Thread | None = None
     if watch_enabled and source_dir is not None:
-        stop_event, watch_thread = _start_watcher(app, source_dir, build_dir, watch_interval)
+        stop_event, watch_thread = _start_watcher(
+            app,
+            source_dir,
+            build_dir,
+            Path(config.server.plugins_dir),
+            watch_interval,
+            compile_fn=compile_fn,
+        )
     elif watch_enabled and source_dir is None:
         print("[webbed-duck] Watch mode enabled but no source directory configured", file=sys.stderr)
 
     host = args.host or config.server.host
     port = args.port or config.server.port
 
+    from webbed_duck.server import preferred_uvicorn_http_implementation
+
     import uvicorn
 
     try:
-        uvicorn.run(app, host=host, port=port)
+        uvicorn.run(
+            app,
+            host=host,
+            port=port,
+            http=preferred_uvicorn_http_implementation(),
+        )
     finally:
         if stop_event is not None:
             stop_event.set()
@@ -193,7 +250,11 @@ def _cmd_serve(args: argparse.Namespace) -> int:
 
 
 def _cmd_run_incremental(args: argparse.Namespace) -> int:
-    config = load_config(args.config)
+    try:
+        config = load_config(Path(args.config))
+    except ConfigError as exc:
+        print(f"[webbed-duck] ERROR: {exc}", file=sys.stderr)
+        return 2
     start = _parse_date(args.start)
     end = _parse_date(args.end)
     results = run_incremental(
@@ -210,7 +271,11 @@ def _cmd_run_incremental(args: argparse.Namespace) -> int:
 
 
 def _cmd_perf(args: argparse.Namespace) -> int:
-    config = load_config(args.config)
+    try:
+        config = load_config(Path(args.config))
+    except ConfigError as exc:
+        print(f"[webbed-duck] ERROR: {exc}", file=sys.stderr)
+        return 2
     params = _parse_param_assignments(args.param)
     iterations = max(1, int(args.iterations))
     timings: list[float] = []
@@ -232,12 +297,28 @@ def _cmd_perf(args: argparse.Namespace) -> int:
     return 0
 
 
-def _start_watcher(app, source_dir: Path, build_dir: Path, interval: float) -> tuple[threading.Event, threading.Thread]:
+def _start_watcher(
+    app,
+    source_dir: Path,
+    build_dir: Path,
+    plugins_dir: Path | None,
+    interval: float,
+    *,
+    compile_fn=compile_routes,
+) -> tuple[threading.Event, threading.Thread]:
     stop_event = threading.Event()
     effective_interval = max(WATCH_INTERVAL_MIN, float(interval))
     thread = threading.Thread(
         target=_watch_source,
-        args=(app, source_dir, build_dir, effective_interval, stop_event),
+        args=(
+            app,
+            source_dir,
+            build_dir,
+            plugins_dir,
+            effective_interval,
+            stop_event,
+            compile_fn,
+        ),
         daemon=True,
         name="webbed-duck-watch",
     )
@@ -248,11 +329,26 @@ def _start_watcher(app, source_dir: Path, build_dir: Path, interval: float) -> t
     return stop_event, thread
 
 
-def _watch_source(app, source_dir: Path, build_dir: Path, interval: float, stop_event: threading.Event) -> None:
-    snapshot = build_source_fingerprint(source_dir)
+def _watch_source(
+    app,
+    source_dir: Path,
+    build_dir: Path,
+    plugins_dir: Path | None,
+    interval: float,
+    stop_event: threading.Event,
+    compile_fn=compile_routes,
+) -> None:
+    snapshot = build_watch_snapshot(source_dir, plugins_dir)
     while not stop_event.wait(interval):
         try:
-            snapshot, count = _watch_iteration(app, source_dir, build_dir, snapshot)
+            snapshot, count = _watch_iteration(
+                app,
+                source_dir,
+                build_dir,
+                plugins_dir,
+                snapshot,
+                compile_fn=compile_fn,
+            )
         except Exception as exc:  # pragma: no cover - runtime safeguard
             print(f"[webbed-duck] Watcher failed to compile routes: {exc}", file=sys.stderr)
             continue
@@ -284,36 +380,67 @@ def _watch_iteration(
     app,
     source_dir: Path,
     build_dir: Path,
-    snapshot: SourceFingerprint,
+    plugins_dir: Path | None,
+    snapshot: WatchSnapshot,
     *,
+    compile_fn=compile_routes,
     compile_and_reload=_compile_and_reload,
-) -> tuple[SourceFingerprint, int]:
+) -> tuple[WatchSnapshot, int]:
     """Perform a single watch iteration and return the updated snapshot and reload count."""
 
-    current = build_source_fingerprint(source_dir)
-    if not snapshot.has_changed(current):
-        return snapshot, 0
-    count = compile_and_reload(app, source_dir, build_dir)
+    current = build_watch_snapshot(source_dir, plugins_dir)
+    plugin_changes = snapshot.plugin_changes(current)
+    if plugin_changes:
+        loader = getattr(app.state, "plugin_loader", None)
+        if loader is not None:
+            for path in sorted(plugin_changes):
+                loader.invalidate(path)
+        if plugin_changes:
+            joined = ", ".join(sorted(plugin_changes))
+            print(f"[webbed-duck] Detected plugin updates: {joined}")
+
+    if not snapshot.routes_changed(current):
+        return current, 0
+    count = compile_and_reload(app, source_dir, build_dir, compile_fn=compile_fn)
     return current, count
 
 
-def build_source_fingerprint(
-    source_dir: Path, *, patterns: Sequence[str] = ("*.toml", "*.sql", "*.md")
-) -> SourceFingerprint:
-    files: dict[str, tuple[float, int]] = {}
+def build_watch_snapshot(
+    source_dir: Path,
+    plugins_dir: Path | None,
+    *,
+    route_patterns: Sequence[str] = ("*.toml", "*.sql", "*.md"),
+) -> WatchSnapshot:
+    routes: dict[str, tuple[float, int]] = {}
     root = Path(source_dir)
-    if not root.exists():
-        return SourceFingerprint(files)
-    for pattern in patterns:
-        for path in sorted(root.rglob(pattern)):
-            if not path.is_file():
-                continue
-            try:
-                stat = path.stat()
-            except FileNotFoundError:  # pragma: no cover - filesystem race
-                continue
-            files[str(path.relative_to(root))] = (stat.st_mtime, stat.st_size)
-    return SourceFingerprint(files)
+    if root.exists():
+        for pattern in route_patterns:
+            for path in sorted(root.rglob(pattern)):
+                if not path.is_file():
+                    continue
+                try:
+                    stat = path.stat()
+                except FileNotFoundError:  # pragma: no cover - filesystem race
+                    continue
+                routes[str(path.relative_to(root))] = (stat.st_mtime, stat.st_size)
+
+    plugins: dict[str, tuple[float, int]] = {}
+    if plugins_dir is not None:
+        plugin_root = Path(plugins_dir)
+        if plugin_root.exists():
+            for path in sorted(plugin_root.rglob("*.py")):
+                if not path.is_file():
+                    continue
+                try:
+                    stat = path.stat()
+                except FileNotFoundError:  # pragma: no cover - filesystem race
+                    continue
+                plugins[str(path.relative_to(plugin_root).as_posix())] = (
+                    stat.st_mtime,
+                    stat.st_size,
+                )
+
+    return WatchSnapshot(routes=routes, plugins=plugins)
 
 
 def _parse_param_assignments(pairs: Sequence[str]) -> Mapping[str, str]:
