@@ -18,9 +18,12 @@ from fastapi.staticfiles import StaticFiles
 
 from .. import __version__ as PACKAGE_VERSION
 from ..config import Config
+from ..runtime.paths import get_storage
 from ..static.chartjs import CHARTJS_VERSION
+from ..core.interpolation import TemplateInterpolationError, render_sql
 from ..core.routes import ParameterSpec, ParameterType, RouteDefinition
 from ..plugins.charts import render_route_charts
+from ..plugins.loader import PluginLoadError, PluginLoader
 from .analytics import AnalyticsStore, ExecutionMetrics
 from .auth import resolve_auth_adapter
 from .cache import CacheStore, parse_invariant_filters
@@ -125,16 +128,26 @@ def create_app(routes: Sequence[RouteDefinition], config: Config) -> FastAPI:
     if not routes:
         raise ValueError("At least one route must be provided to create the application")
 
+    storage_root = get_storage(config)
+
+    try:
+        plugin_loader = PluginLoader(config.server.plugins_dir)
+    except PluginLoadError as exc:
+        raise RuntimeError(f"Failed to initialise plugins: {exc}") from exc
+
     app = FastAPI(title="webbed_duck", version=PACKAGE_VERSION)
     app.state.config = config
+    app.state.storage_root = storage_root
+    app.state.plugin_loader = plugin_loader
+    app.state.plugins_dir = plugin_loader.root
     app.state.analytics = AnalyticsStore(
         weight=config.analytics.weight_interactions,
         enabled=config.analytics.enabled,
     )
     app.state.routes = list(routes)
     app.state.route_index = {route.id: route for route in app.state.routes}
-    app.state.overlays = OverlayStore(config.server.storage_root)
-    app.state.meta = MetaStore(config.server.storage_root)
+    app.state.overlays = OverlayStore(storage_root)
+    app.state.meta = MetaStore(storage_root)
     app.state.session_store = SessionStore(app.state.meta, config.auth)
     app.state.share_store = ShareStore(app.state.meta, config)
     app.state.email_sender = load_email_sender(config.email.adapter)
@@ -144,9 +157,9 @@ def create_app(routes: Sequence[RouteDefinition], config: Config) -> FastAPI:
         session_store=app.state.session_store,
     )
 
-    app.state.cache_store = CacheStore(config.server.storage_root)
+    app.state.cache_store = CacheStore(storage_root)
     chartjs_route = f"/vendor/{CHARTJS_FILENAME}"
-    _prepare_chartjs_assets(app, config.server.storage_root)
+    _prepare_chartjs_assets(app, storage_root)
     app.state._dynamic_route_handles = _register_dynamic_routes(app, app.state.routes)
 
     assets_root = Path(__file__).resolve().parent.parent / "static" / "assets" / "wd"
@@ -249,7 +262,24 @@ def create_app(routes: Sequence[RouteDefinition], config: Config) -> FastAPI:
         route = _get_route(app.state.routes, route_id)
         params = _collect_params(route, request)
         ordered = [_value_for_name(params, name, route) for name in route.param_order]
-        table = _execute_sql(_limit_zero(route.prepared_sql), ordered)
+        bindings: dict[str, object] = {}
+        for name, value in zip(route.param_order, ordered):
+            bindings[name] = value
+        for name, value in route.constant_params.items():
+            bindings[name] = value
+        try:
+            rendered_sql = render_sql(
+                route,
+                params,
+                config=request.app.state.config.interpolation,
+                request=request,
+            )
+        except TemplateInterpolationError as exc:
+            raise HTTPException(
+                status_code=400,
+                detail={"code": "invalid_template", "message": str(exc)},
+            ) from exc
+        table = _execute_sql(_limit_zero(rendered_sql), bindings)
         schema = [
             {"name": field.name, "type": str(field.type)}
             for field in table.schema
@@ -336,7 +366,7 @@ def create_app(routes: Sequence[RouteDefinition], config: Config) -> FastAPI:
         record = {column: payload.get(column) for column in columns}
         try:
             path = append_record(
-                app.state.config.server.storage_root,
+                app.state.storage_root,
                 destination=destination,
                 columns=columns,
                 record=record,
@@ -714,7 +744,14 @@ def _execute_route(
     offset: int | None,
     limit: int | None,
 ) -> RouteExecutionResult:
-    processed = run_preprocessors(route.preprocess, params, route=route, request=request)
+    plugin_loader: PluginLoader = getattr(request.app.state, "plugin_loader")
+    processed = run_preprocessors(
+        route.preprocess,
+        params,
+        route=route,
+        request=request,
+        loader=plugin_loader,
+    )
     ordered = [_value_for_name(processed, name, route) for name in route.param_order]
     sanitized_offset = max(0, offset or 0)
     sanitized_limit = None if limit is None else max(0, int(limit))
@@ -723,6 +760,7 @@ def _execute_route(
         getattr(request.app.state, "route_index", {}),
         cache_store=cache_store,
         config=request.app.state.config,
+        plugin_loader=plugin_loader,
     )
     start = time.perf_counter()
     try:
@@ -1365,7 +1403,7 @@ def _coerce_int(value: object, label: str) -> int | None:
     raise _http_error("invalid_parameter", f"{label} must be an integer")
 
 
-def _execute_sql(sql: str, params: Iterable[object]) -> pa.Table:
+def _execute_sql(sql: str, params: Mapping[str, object]) -> pa.Table:
     con = duckdb.connect()
     try:
         cursor = con.execute(sql, params)
