@@ -1,13 +1,14 @@
 from __future__ import annotations
 
 import base64
+import datetime as _dt
+import decimal
 import hashlib
 import json
+import math
 import shutil
 import time
 from dataclasses import dataclass, field, replace
-import decimal
-import math
 from itertools import product
 from pathlib import Path
 from typing import Callable, Mapping, Sequence
@@ -20,6 +21,12 @@ import pyarrow.compute as pc
 
 from ..config import CacheConfig
 from ..core.routes import RouteDefinition
+from ..utils.datetime import isoformat_datetime, parse_iso_date, parse_iso_datetime
+
+try:  # pragma: no cover - Python < 3.9 fallback
+    from zoneinfo import ZoneInfo
+except ImportError:  # pragma: no cover - Python < 3.9
+    ZoneInfo = None  # type: ignore[assignment]
 
 
 RecordBatchFactory = Callable[[], tuple[pa.RecordBatchReader, Callable[[], None]]]
@@ -111,6 +118,7 @@ class CacheStore:
             "rows_per_page": settings.rows_per_page,
             "order_by": list(settings.order_by),
             "params": _normalize_mapping(params),
+            "constants": _constant_snapshot(route.constants, route.constant_types),
             "invariants": [
                 {
                     "param": setting.param,
@@ -773,6 +781,10 @@ def _coerce_invariant_value_for_column(
             return _coerce_float_like(value)
         if pa.types.is_decimal(column_type):
             return _coerce_decimal_like(value)
+        if pa.types.is_date(column_type):
+            return _coerce_date_like(value)
+        if pa.types.is_timestamp(column_type):
+            return _coerce_timestamp_like(value, column_type)
     except (TypeError, ValueError, decimal.InvalidOperation):
         return _INVALID_INVARIANT_VALUE
     return value
@@ -850,6 +862,44 @@ def _coerce_decimal_like(value: object) -> decimal.Decimal:
     return decimal_value
 
 
+def _coerce_date_like(value: object) -> _dt.date:
+    if isinstance(value, _dt.datetime):
+        return value.date()
+    if isinstance(value, _dt.date):
+        return value
+    if isinstance(value, (bytes, bytearray)):
+        text = bytes(value).decode("utf-8")
+    else:
+        text = str(value)
+    return parse_iso_date(text)
+
+
+def _coerce_timestamp_like(value: object, column_type: pa.DataType) -> _dt.datetime:
+    if isinstance(value, _dt.datetime):
+        dt_value = value
+    elif isinstance(value, _dt.date):
+        dt_value = _dt.datetime.combine(value, _dt.time())
+    elif isinstance(value, (bytes, bytearray)):
+        dt_value = parse_iso_datetime(bytes(value).decode("utf-8"))
+    else:
+        dt_value = parse_iso_datetime(str(value))
+
+    timezone_name = getattr(column_type, "tz", None)
+    if timezone_name and ZoneInfo is not None:
+        try:
+            zone = ZoneInfo(timezone_name)
+        except Exception:  # pragma: no cover - invalid/unknown zone
+            zone = None
+        if zone is not None:
+            if dt_value.tzinfo is None:
+                dt_value = dt_value.replace(tzinfo=zone)
+            else:
+                dt_value = dt_value.astimezone(zone)
+    elif not timezone_name and dt_value.tzinfo is not None:
+        dt_value = dt_value.astimezone(_dt.timezone.utc).replace(tzinfo=None)
+    return dt_value
+
+
 def _prepare_invariant_filter_values(
     values: Sequence[object],
     column: pa.ChunkedArray,
@@ -895,6 +945,10 @@ def canonicalize_invariant_value(value: object, setting: InvariantFilterSetting)
         token = _canonicalize_numeric_token(value)
     elif isinstance(value, (bytes, bytearray)):
         token = "bytes:" + base64.b64encode(bytes(value)).decode("ascii")
+    elif isinstance(value, _dt.datetime):
+        token = "datetime:" + isoformat_datetime(value)
+    elif isinstance(value, _dt.date):
+        token = "date:" + value.isoformat()
     elif hasattr(value, "isoformat"):
         try:
             token = "datetime:" + value.isoformat()
@@ -1114,7 +1168,7 @@ def resolve_cache_settings(route: RouteDefinition, config: CacheConfig) -> Cache
 def fetch_cached_table(
     route: RouteDefinition,
     params: Mapping[str, object],
-    ordered_params: Sequence[object],
+    bound_params: Mapping[str, object],
     *,
     offset: int,
     limit: int | None,
@@ -1126,8 +1180,8 @@ def fetch_cached_table(
     settings = resolve_cache_settings(route, config)
     effective_offset, effective_limit = _effective_window(offset, limit, settings)
     invariant_requests = _collect_invariant_requests(settings.invariant_filters, params)
-    reader_factory_fn = reader_factory or (lambda: _record_batch_reader(route.prepared_sql, ordered_params))
-    execute_sql_fn = execute_sql or (lambda: _execute_sql(route.prepared_sql, ordered_params))
+    reader_factory_fn = reader_factory or (lambda: _record_batch_reader(route.prepared_sql, bound_params))
+    execute_sql_fn = execute_sql or (lambda: _execute_sql(route.prepared_sql, bound_params))
 
     if not store or not settings.enabled:
         table = execute_sql_fn()
@@ -1223,7 +1277,7 @@ def fetch_cached_table(
 def materialize_parquet_artifacts(
     route: RouteDefinition,
     params: Mapping[str, object],
-    ordered_params: Sequence[object],
+    bound_params: Mapping[str, object],
     *,
     store: CacheStore | None,
     config: CacheConfig,
@@ -1243,7 +1297,7 @@ def materialize_parquet_artifacts(
         raise RuntimeError(
             f"Route '{route.id}' received invariant-filter overrides incompatible with parquet_path mode"
         )
-    reader_factory_fn = reader_factory or (lambda: _record_batch_reader(route.prepared_sql, ordered_params))
+    reader_factory_fn = reader_factory or (lambda: _record_batch_reader(route.prepared_sql, bound_params))
     cache_params = _prepare_cache_params(params, settings.invariant_filters)
     key = store.compute_key(route, cache_params, settings)
     route_signature = _route_signature(route)
@@ -1513,7 +1567,7 @@ def _slice_table(
 
 def _record_batch_reader(
     sql: str,
-    params: Sequence[object],
+    params: Mapping[str, object],
 ) -> tuple[pa.RecordBatchReader, Callable[[], None]]:
     con = duckdb.connect()
     cursor = con.execute(sql, params)
@@ -1521,7 +1575,7 @@ def _record_batch_reader(
     return reader, con.close
 
 
-def _execute_sql(sql: str, params: Sequence[object]) -> pa.Table:
+def _execute_sql(sql: str, params: Mapping[str, object]) -> pa.Table:
     con = duckdb.connect()
     try:
         cursor = con.execute(sql, params)
@@ -1550,6 +1604,7 @@ def _route_signature(route: RouteDefinition) -> str:
         "version": route.version,
         "sql": route.prepared_sql,
         "order": list(route.param_order),
+        "constants": _constant_snapshot(route.constants, route.constant_types),
     }
     encoded = json.dumps(payload, sort_keys=True, default=_json_default).encode("utf-8")
     return hashlib.sha256(encoded).hexdigest()
@@ -1571,6 +1626,10 @@ def _normalize_mapping(values: Mapping[str, object]) -> dict[str, object]:
 
 
 def _json_default(value: object) -> object:
+    if isinstance(value, _dt.datetime):
+        return isoformat_datetime(value)
+    if isinstance(value, _dt.date):
+        return value.isoformat()
     if hasattr(value, "isoformat"):
         try:
             return value.isoformat()
@@ -1581,6 +1640,52 @@ def _json_default(value: object) -> object:
     return value
 
 
+def _constant_snapshot(
+    values: Mapping[str, object], types: Mapping[str, str]
+) -> list[dict[str, object]]:
+    snapshot: list[dict[str, object]] = []
+    for name in sorted(values):
+        type_name = types.get(name)
+        snapshot.append(
+            {
+                "name": name,
+                "type": type_name,
+                "value": _constant_json_value(values[name], type_name),
+            }
+        )
+    return snapshot
+
+
+def _constant_json_value(value: object, type_name: str | None) -> object:
+    normalized = type_name.upper() if type_name else None
+    if normalized == "BOOLEAN":
+        return bool(value)
+    if normalized == "DATE":
+        if isinstance(value, _dt.date) and not isinstance(value, _dt.datetime):
+            return value.isoformat()
+        return str(value)
+    if normalized == "TIMESTAMP":
+        if isinstance(value, _dt.datetime):
+            return value.isoformat()
+        return str(value)
+    if normalized == "DECIMAL":
+        return str(value)
+    if normalized == "INTEGER":
+        return int(value)
+    if normalized == "DOUBLE":
+        return float(value)
+    if normalized == "IDENTIFIER":
+        return str(value)
+    if isinstance(value, (bool, int, float)):
+        return value
+    if hasattr(value, "isoformat"):
+        try:
+            return value.isoformat()
+        except Exception:  # pragma: no cover - defensive
+            return str(value)
+    return str(value)
+
+
 __all__ = [
     "CacheKey",
     "CacheQueryResult",
